from macros.base_prod import STOCKPILE_PER_POP
from macros.priorities import (
    AFTER_ALL_TARGET_MAX_METERS_PRIORITY,
    TARGET_EARLY_BEFORE_SCALING_PRIORITY,
)

try:
    from focs._effects import (
        Abs,
        CurrentTurn,
        EffectsGroup,
        Focus,
        HasSpecies,
        Homeworld,
        IsSource,
        LocalCandidate,
        MinOf,
        OwnedBy,
        Planet,
        SetMaxStockpile,
        SetStockpile,
        Source,
        Target,
        TargetPopulation,
        Value,
    )
except ModuleNotFoundError:
    pass


def STOCKPILE_PER_POP_EFFECTSGROUP__SNIP(label: str, value) -> dict:
    return {
        "scope": IsSource,
        "activation": Planet(),
        "accountinglabel": label + "_STOCKPILE_LABEL",
        "priority": TARGET_EARLY_BEFORE_SCALING_PRIORITY,
        "effects": SetMaxStockpile(value=value),
    }


STANDARD_STOCKPILE = [
    EffectsGroup(  # increase or decrease towards target meter, when not recently conquered
        scope=IsSource,
        activation=Planet()
        & (LocalCandidate.LastTurnConquered < CurrentTurn)
        & (LocalCandidate.System.LastTurnBattleHere < CurrentTurn),
        priority=AFTER_ALL_TARGET_MAX_METERS_PRIORITY,
        effects=SetStockpile(
            value=Value
            + MinOf(float, Abs(float, Value(Target.MaxStockpile) - Value), 1)
            * (1 - 2 * (Value > Value(Target.MaxStockpile)))
        ),
    ),
    # increase stockpile for species if Homeworld is set to stockpile focus
    EffectsGroup(
        scope=Planet()
        & OwnedBy(empire=Source.Owner)
        & HasSpecies(name=[Source.Species])
        & ~Homeworld(name=[Source.Species]),
        activation=Planet() & Focus(type=["FOCUS_STOCKPILE"]) & Homeworld(),
        stackinggroup="HOMEWORLD_STOCKPILE_FOCUS_BONUS_LABEL",
        accountinglabel="HOMEWORLD_STOCKPILE_FOCUS_BONUS_LABEL",
        effects=SetMaxStockpile(value=(Value + 2 * Target.Population * STOCKPILE_PER_POP)),
    ),
    # removes residual stockpile capacity from a dead planet
    EffectsGroup(scope=IsSource, activation=Planet() & TargetPopulation(high=0), effects=SetStockpile(value=0)),
]

# NO_STOCKPILE
# '''
# '''

# BAD_STOCKPILE
# '''[[STOCKPILE_PER_POP_EFFECTSGROUP(BAD,Value     +  0.5 * Target.Population * [[STOCKPILE_PER_POP]])]]
# [[STANDARD_STOCKPILE]]
# '''


AVERAGE_STOCKPILE = [
    EffectsGroup(
        # Skip the AVERAGE_STOCKPILE_DESC, same as for the other *_STOCKPILE macros
        **STOCKPILE_PER_POP_EFFECTSGROUP__SNIP("AVERAGE", Value + 1 * Target.Population * STOCKPILE_PER_POP)
    ),
    *STANDARD_STOCKPILE,
]


# GOOD_STOCKPILE
# '''[[STOCKPILE_PER_POP_EFFECTSGROUP(GOOD,Value    +  3 * Target.Population * [[STOCKPILE_PER_POP]])]]
# [[STANDARD_STOCKPILE]]
# '''

# GREAT_STOCKPILE
# '''[[STOCKPILE_PER_POP_EFFECTSGROUP(GREAT,Value  +  10 * Target.Population * [[STOCKPILE_PER_POP]])]]
# [[STANDARD_STOCKPILE]]
# '''

# ULTIMATE_STOCKPILE
# '''[[STOCKPILE_PER_POP_EFFECTSGROUP(ULTIMATE, Value + 15 * Target.Population * [[STOCKPILE_PER_POP]])]]
# [[STANDARD_STOCKPILE]]
# '''

# STOCKPILE_PER_POP_EFFECTSGROUP
# '''     EffectsGroup
#             description = "@1@_STOCKPILE_DESC"
#         [[STOCKPILE_PER_POP_EFFECTSGROUP__SNIP(@1@,@2@)]]
# '''


# #include "../macros/base_prod.macros"
# #include "../macros/priorities.macros"



#!/usr/bin/env python3


import os
import sys
from datetime import datetime, timezone
from glob import glob
from platform import system
from string import Template
from subprocess import CalledProcessError, check_output

INVALID_BUILD_NO = "???"


class Generator:
    def __init__(self, infile, outfile):
        self.infile = infile
        self.outfile = outfile

    def compile_output(self, template, version, branch, build_no, build_sys, version_file_name):
        return template.substitute(
            FreeOrion_VERSION=version,
            FreeOrion_BRANCH=branch,
            FreeOrion_BUILD_NO=build_no,
            FreeOrion_BUILDSYS=build_sys,
            FreeOrion_VERSION_FILENAME=version_file_name,
        )

    def execute(self, version, branch, build_no, build_sys, version_file_name):
        if build_no == INVALID_BUILD_NO:
            print("WARNING: Can't determine git commit!")

        if os.path.isfile(self.outfile):
            with open(self.outfile) as check_file:
                check_file_contents = check_file.read()
                if build_no == INVALID_BUILD_NO:
                    if version in check_file_contents:
                        print("Version matches version in existing Version.cpp, skip regenerating it")
                        return
                elif build_no in check_file_contents:
                    print("Build number matches build number in existing Version.cpp, skip regenerating it")
                    return

        try:
            with open(self.infile) as template_file:
                template = Template(template_file.read())
        except OSError:
            print(f"WARNING: Can't access {self.infile}, {self.outfile} not updated!")
            return

        print("Writing file: %s" % self.outfile)
        with open(self.outfile, "w") as generated_file:
            generated_file.write(self.compile_output(template, version, branch, build_no, build_sys, version_file_name))


class NsisInstScriptGenerator(Generator):
    def compile_dll_list(self):
        all_dll_files = glob("*.dll")
        accepted_dll_files = {"GiGi.dll"}
        for dll_file in all_dll_files:
            if dll_file.startswith("boost_"):
                if dll_file.partition(".")[0] in required_boost_libraries:
                    accepted_dll_files.add(dll_file)
            else:
                accepted_dll_files.add(dll_file)
        return sorted(accepted_dll_files)

    def compile_output(self, template, version, branch, build_no, build_sys, version_file_name):
        dll_files = self.compile_dll_list()
        if dll_files:
            return template.substitute(
                FreeOrion_VERSION=version,
                FreeOrion_BRANCH=branch,
                FreeOrion_BUILD_NO=build_no,
                FreeOrion_BUILDSYS=build_sys,
                FreeOrion_DLL_LIST_INSTALL="\n  ".join(['File "..\\' + fname + '"' for fname in dll_files]),
                FreeOrion_DLL_LIST_UNINSTALL="\n  ".join(['Delete "$INSTDIR\\' + fname + '"' for fname in dll_files]),
                FreeOrion_PYTHON_VERSION="%d%d" % (sys.version_info.major, sys.version_info.minor),
            )

        else:
            print("WARNING: no dll files for installer package found")
            return template.substitute(
                FreeOrion_VERSION=version,
                FreeOrion_BRANCH=branch,
                FreeOrion_BUILD_NO=build_no,
                FreeOrion_BUILDSYS=build_sys,
                FreeOrion_DLL_LIST_INSTALL="",
                FreeOrion_DLL_LIST_UNINSTALL="",
                FreeOrion_PYTHON_VERSION="",
            )


if len(sys.argv) not in (3, 4):
    print("ERROR: invalid parameters.")
    print("make_versioncpp.py <project rootdir> <build system name> [<boost python suffix>]")
    quit()

os.chdir(sys.argv[1])
build_sys = sys.argv[2]
if 4 == len(sys.argv):
    boost_python_suffix = sys.argv[3]
else:
    boost_python_suffix = "%d%d" % (sys.version_info.major, sys.version_info.minor)

required_boost_libraries = [
    "boost_atomic",
    "boost_chrono",
    "boost_date_time",
    "boost_filesystem",
    "boost_iostreams",
    "boost_locale",
    "boost_log",
    "boost_log_setup",
    "boost_python%s" % boost_python_suffix,
    "boost_regex",
    "boost_serialization",
    "boost_signals",
    "boost_system",
    "boost_thread",
]


# A list of tuples containing generators
generators = [
    Generator("util/Version.cpp.in", "util/Version.cpp"),
    Generator("cmake/FreeOrionVersion.cmake.in", "cmake/FreeOrionVersion.cmake"),
]
if system() == "Windows":
    generators.append(NsisInstScriptGenerator("packaging/windows_installer.nsi.in", "packaging/windows_installer.nsi"))
if system() == "Darwin":
    generators.append(Generator("packaging/Info.plist.in", "packaging/Info.plist"))

version = "0.5.1.2"
branch = ""
build_no = INVALID_BUILD_NO
version_file_name = version

try:
    branch = check_output(["git", "rev-parse", "--abbrev-ref", "HEAD"], text=True).strip()
    if (branch == "master") or (branch == "weekly-test-builds") or (branch[:7] == "release"):
        branch = ""
    else:
        branch += " "
    commit = check_output(
        ["git", "show", "--no-show-signature", "-s", "--format=%h", "--abbrev=7", "HEAD"], text=True
    ).strip()
    timestamp = float(
        check_output(["git", "show", "--no-show-signature", "-s", "--format=%ct", "HEAD"], text=True).strip()
    )
    build_no = ".".join([datetime.fromtimestamp(timestamp, tz=timezone.utc).strftime("%Y-%m-%d"), commit])
    if branch[:7] == "release":
        version_file_name = "v" + version
    else:
        version_file_name = build_no + "_Test"
except (OSError, CalledProcessError):
    print("WARNING: git not installed or not setup correctly")

for generator in generators:
    generator.execute(version, branch, build_no, build_sys, version_file_name)

print(f"Building v{version} {branch}build {build_no}")



from macros.misc import SUPPLY_DISCONNECTED_INFLUENCE_MALUS
from macros.priorities import (
    TARGET_EARLY_BEFORE_SCALING_PRIORITY,
    TARGET_LAST_BEFORE_OVERRIDE_PRIORITY,
    TARGET_SCALING_PRIORITY,
)

try:
    from focs._effects import (
        Abs,
        Capital,
        EffectsGroup,
        EmpireHasAdoptedPolicy,
        Focus,
        GalaxyMaxAIAggression,
        Happiness,
        HasSpecies,
        HasTag,
        Homeworld,
        IsHuman,
        IsSource,
        LocalCandidate,
        NamedReal,
        OwnedBy,
        Planet,
        ResourceSupplyConnected,
        SetTargetInfluence,
        Source,
        StatisticCount,
        Target,
        Unowned,
        Value,
    )
except ModuleNotFoundError:
    pass


BASE_INFLUENCE_COSTS = [
    EffectsGroup(  # colonies consume influence, proportional to square-root of how many populated planets and non-populated outposts the empire controls
        scope=IsSource,
        activation=Planet() & ~Unowned & ~Capital,
        stackinggroup="IMPERIAL_PALACE_MANY_PLANETS_INFLUENCE_PENALTY",
        accountinglabel="COLONY_ADMIN_COSTS_LABEL",
        priority=TARGET_LAST_BEFORE_OVERRIDE_PRIORITY,
        effects=SetTargetInfluence(
            value=Value
            - NamedReal(name="COLONY_ADMIN_COSTS_PER_PLANET", value=0.4)
            * (
                (
                    StatisticCount(
                        float,
                        condition=Planet()
                        & OwnedBy(empire=Source.Owner)
                        & HasSpecies()
                        & ~HasSpecies(name=["SP_EXOBOT"]),
                    )
                    + (
                        NamedReal(name="OUTPOST_RELATIVE_ADMIN_COUNT", value=0.25)
                        * (
                            StatisticCount(
                                float,
                                condition=Planet()
                                & OwnedBy(empire=Source.Owner)
                                & (~HasSpecies() | HasSpecies(name=["SP_EXOBOT"])),
                            )
                        )
                    )
                )
                ** 0.5
            )
        ),
    ),
    EffectsGroup(  # species homeworlds consume more influence
        scope=IsSource,
        activation=Planet() & ~Unowned & ~Capital & Homeworld(name=[LocalCandidate.Species]),
        accountinglabel="SPECIES_HOMEWORLD_INDEPENDENCE_DESIRE_LABEL",
        priority=TARGET_LAST_BEFORE_OVERRIDE_PRIORITY,
        effects=SetTargetInfluence(value=Value - 1),
    ),
    EffectsGroup(  # colonies consume more influence if not supply connected to empire's capital
        scope=IsSource,
        activation=Planet()
        & ~Unowned
        & ~EmpireHasAdoptedPolicy(empire=Source.Owner, name="PLC_CONFEDERATION")
        & ~ResourceSupplyConnected(
            empire=LocalCandidate.Owner, condition=Planet() & OwnedBy(empire=Source.Owner) & Capital
        ),
        accountinglabel="CAPITAL_DISCONNECTION_LABEL",
        priority=TARGET_LAST_BEFORE_OVERRIDE_PRIORITY,
        effects=SetTargetInfluence(
            value=Value
            - NamedReal(name="SUPPLY_DISCONNECTED_INFLUENCE_MALUS", value=SUPPLY_DISCONNECTED_INFLUENCE_MALUS)
        ),
    ),
    EffectsGroup(  # gives human bonuses when AI Aggression set to Beginner
        scope=IsSource,
        activation=Planet() & ~Unowned & IsHuman & (GalaxyMaxAIAggression == 0),  # human player, not human species
        accountinglabel="DIFFICULTY",
        priority=TARGET_LAST_BEFORE_OVERRIDE_PRIORITY,
        effects=SetTargetInfluence(value=Value + 1),
    ),
]

# NO_INFLUENCE
# '''[[DESCRIPTION_EFFECTSGROUP_MACRO(NO_INFLUENCE_DESC)]]
# [[BASE_INFLUENCE_COSTS]]
# '''

ARTISANS_INFLUENCE_STABILITY = [
    EffectsGroup(  # artistic species generate influence when artisans workshops policy adopted
        scope=IsSource,
        activation=HasTag(name="ARTISTIC")
        & Happiness(low=NamedReal(name="ARTISANS_MIN_STABILITY_FOCUS", value=1))
        & EmpireHasAdoptedPolicy(empire=Source.Owner, name="PLC_ARTISAN_WORKSHOPS")
        & Focus(type=["FOCUS_INFLUENCE"]),
        accountinglabel="PLC_ARTISAN_WORKSHOPS",
        effects=SetTargetInfluence(value=Value + NamedReal(name="ARTISANS_INFLUENCE_FLAT_FOCUS", value=2.0)),
    ),
    EffectsGroup(
        scope=IsSource,
        activation=HasTag(name="ARTISTIC")
        & Happiness(low=NamedReal(name="ARTISANS_MIN_STABILITY_NO_FOCUS", value=10))
        & EmpireHasAdoptedPolicy(empire=Source.Owner, name="PLC_ARTISAN_WORKSHOPS")
        & ~Focus(type=["FOCUS_INFLUENCE"]),
        accountinglabel="PLC_ARTISAN_WORKSHOPS",
        effects=SetTargetInfluence(value=Value + NamedReal(name="ARTISANS_INFLUENCE_FLAT_NO_FOCUS", value=0.5)),
    ),
]

BASIC_INFLUENCE = [
    EffectsGroup(  # influence focus generates influence from planets, proportional to sqare-root of population
        scope=IsSource,
        activation=Planet() & Focus(type=["FOCUS_INFLUENCE"]),
        accountinglabel="FOCUS_INFLUENCE_LABEL",
        priority=TARGET_EARLY_BEFORE_SCALING_PRIORITY,
        effects=SetTargetInfluence(
            value=Value
            + (Target.Population**0.5) * NamedReal(name="FOCUS_INFLUENCE_INFLUENCE_PER_SQRT_POP", value=1.0)
        ),
    ),
    *BASE_INFLUENCE_COSTS,
    *ARTISANS_INFLUENCE_STABILITY,
]

# VERY_BAD_INFLUENCE
# '''[[BASIC_INFLUENCE]]

#         EffectsGroup
#             description = "VERY_BAD_INFLUENCE_DESC"
#             scope = Source
#             activation = And [
#                 Planet
#                 Focus type = "FOCUS_INFLUENCE"
#             ]
#             accountinglabel = "VERY_BAD_INFLUENCE_LABEL"
#             priority = [[TARGET_SCALING_PRIORITY]]
#             effects = SetTargetInfluence value = Value + (2.0/3.0-1.0)*abs(Value)
# '''

# /*
# // Influence adjustment calculation examples...

# Value  Multiplier       Calc            Adjustment
# 5.0       1.2       (1.2-1)*abs(5.0)        1.0
# 0.0       1.2       (1.2-1)*abs(0.0)        0.0
# -5.0      1.2       (1.2-1)*abs(-5.0)       1.0
# 5.0       1.0       (1.0-1)*abs(5.0)         0
# 0.0       1.0       (1.0-1)*abs(0.0)         0
# -5.0      1.0       (1.0-1)*abs(-5.0)        0
# 5.0       0.8       (0.8-1)*abs(5.0)       -1.0
# 0.0       0.8       (0.8-1)*abs(0.0)         0
# -5.0      0.8       (0.8-1)*abs(-5.0)      -1.0
# */


# BAD_INFLUENCE
# '''[[BASIC_INFLUENCE]]

#         EffectsGroup
#             description = "BAD_INFLUENCE_DESC"
#             scope = Source
#             activation = And [
#                 Planet
#                 Focus type = "FOCUS_INFLUENCE"
#             ]
#             accountinglabel = "BAD_INFLUENCE_LABEL"
#             priority = [[TARGET_SCALING_PRIORITY]]
#             effects = SetTargetInfluence value = Value + (0.8-1.0)*abs(Value)
# '''

# AVERAGE_INFLUENCE
# '''[[BASIC_INFLUENCE]]
# '''

# GOOD_INFLUENCE
# '''[[BASIC_INFLUENCE]]

#         EffectsGroup
#             description = "GOOD_INFLUENCE_DESC"
#             scope = Source
#             activation = And [
#                 Planet
#                 Focus type = "FOCUS_INFLUENCE"
#             ]
#             accountinglabel = "GOOD_INFLUENCE_LABEL"
#             priority = [[TARGET_SCALING_PRIORITY]]
#             effects = SetTargetInfluence value = Value + (1.25-1.0)*abs(Value)
# '''

GREAT_INFLUENCE = [
    *BASIC_INFLUENCE,
    EffectsGroup(
        description="GREAT_INFLUENCE_DESC",
        scope=IsSource,
        activation=Planet() & Focus(type=["FOCUS_INFLUENCE"]),
        accountinglabel="GREAT_INFLUENCE_LABEL",
        priority=TARGET_SCALING_PRIORITY,
        effects=SetTargetInfluence(value=Value + (1.5 - 1.0) * Abs(float, Value)),
    ),
]

# ULTIMATE_INFLUENCE
# '''[[BASIC_INFLUENCE]]

#         EffectsGroup
#             description = "ULTIMATE_INFLUENCE_DESC"
#             scope = Source
#             activation = And [
#                 Planet
#                 Focus type = "FOCUS_INFLUENCE"
#             ]
#             accountinglabel = "ULTIMATE_INFLUENCE_LABEL"
#             priority = [[TARGET_SCALING_PRIORITY]]
#             effects = SetTargetInfluence value = Value + (2.0-1.0)*abs(Value)
# '''

# #include "../macros/misc.macros"



from macros.misc import SHIP_SHIELD_FACTOR
from macros.priorities import AFTER_ALL_TARGET_MAX_METERS_PRIORITY, DEFAULT_PRIORITY

try:
    from focs._effects import (
        CurrentTurn,
        EffectsGroup,
        GalaxyMaxAIAggression,
        IsHuman,
        IsSource,
        LocalCandidate,
        SetMaxShield,
        SetShield,
        Ship,
        Target,
        Value,
    )
except ModuleNotFoundError:
    pass


STANDARD_SHIP_SHIELDS = [
    EffectsGroup(  # gives human bonuses when AI Aggression set to Beginner
        scope=IsSource,
        activation=IsHuman & (GalaxyMaxAIAggression == 0) & Ship,  # human player, not human species
        accountinglabel="DIFFICULTY",
        priority=DEFAULT_PRIORITY,
        effects=SetMaxShield(value=Value + (1 * SHIP_SHIELD_FACTOR)),
    ),
    EffectsGroup(  # increase to max when not in battle
        scope=IsSource,
        activation=Ship & (LocalCandidate.LastTurnActiveInBattle < CurrentTurn),
        stackinggroup="SHIELD_REGENERATION_EFFECT",
        priority=AFTER_ALL_TARGET_MAX_METERS_PRIORITY,
        effects=SetShield(value=Target.MaxShield),
    ),
]

# BAD_SHIP_SHIELDS
# '''[[STANDARD_SHIP_SHIELDS]]

# EffectsGroup
#             description = "BAD_SHIP_SHIELD_DESC"
#             scope = And [
#                 Source
#                 Ship
#             ]
#             effects = SetMaxShield value = Value
#                 - (0.5 * [[SHIP_SHIELD_FACTOR]] * (Statistic If condition = And [ Source DesignHasPart name = "SH_DEFENSE_GRID"]))
#                 - (1.0 * [[SHIP_SHIELD_FACTOR]] * (Statistic If condition = And [ Source DesignHasPart name = "SH_DEFLECTOR"]))
#                 - (1.5 * [[SHIP_SHIELD_FACTOR]] * (Statistic If condition = And [ Source DesignHasPart name = "SH_PLASMA"]))
#                 - (2.0 * [[SHIP_SHIELD_FACTOR]] * (Statistic If condition = And [ Source DesignHasPart name = "SH_MULTISPEC"]))
#                 - (2.5 * [[SHIP_SHIELD_FACTOR]] * (Statistic If condition = And [ Source DesignHasPart name = "SH_BLACK"]))
# '''

# GOOD_SHIP_SHIELDS
# '''[[STANDARD_SHIP_SHIELDS]]

# EffectsGroup
#             description = "GOOD_SHIP_SHIELD_DESC"
#             scope = And [
#                 Source
#                 Ship
#             ]
#             effects = SetMaxShield value = Value
#                 + (0.5 * [[SHIP_SHIELD_FACTOR]] * (Statistic If condition = And [ Source DesignHasPart name = "SH_DEFENSE_GRID"]))
#                 + (1.0 * [[SHIP_SHIELD_FACTOR]] * (Statistic If condition = And [ Source DesignHasPart name = "SH_DEFLECTOR"]))
#                 + (1.5 * [[SHIP_SHIELD_FACTOR]] * (Statistic If condition = And [ Source DesignHasPart name = "SH_PLASMA"]))
#                 + (2.0 * [[SHIP_SHIELD_FACTOR]] * (Statistic If condition = And [ Source DesignHasPart name = "SH_MULTISPEC"]))
#                 + (2.5 * [[SHIP_SHIELD_FACTOR]] * (Statistic If condition = And [ Source DesignHasPart name = "SH_BLACK"]))
# '''

# GREAT_SHIP_SHIELDS
# '''[[STANDARD_SHIP_SHIELDS]]

# EffectsGroup
#             description = "GREAT_SHIP_SHIELD_DESC"
#             scope = And [
#                 Source
#                 Ship
#             ]
#             effects = SetMaxShield value = Value
#                 + (1.0 * [[SHIP_SHIELD_FACTOR]] * (Statistic If condition = And [ Source DesignHasPart name = "SH_DEFENSE_GRID"]))
#                 + (2.0 * [[SHIP_SHIELD_FACTOR]] * (Statistic If condition = And [ Source DesignHasPart name = "SH_DEFLECTOR"]))
#                 + (3.0 * [[SHIP_SHIELD_FACTOR]] * (Statistic If condition = And [ Source DesignHasPart name = "SH_PLASMA"]))
#                 + (4.0 * [[SHIP_SHIELD_FACTOR]] * (Statistic If condition = And [ Source DesignHasPart name = "SH_MULTISPEC"]))
#                 + (5.0 * [[SHIP_SHIELD_FACTOR]] * (Statistic If condition = And [ Source DesignHasPart name = "SH_BLACK"]))
# '''

# ULTIMATE_SHIP_SHIELDS
# '''[[STANDARD_SHIP_SHIELDS]]

# EffectsGroup
#             description = "ULTIMATE_SHIP_SHIELD_DESC"
#             scope = And [
#                 Source
#                 Ship
#             ]
#             effects = SetMaxShield value = Value
#                 + (1.5 * [[SHIP_SHIELD_FACTOR]] * (Statistic If condition = And [ Source DesignHasPart name = "SH_DEFENSE_GRID"]))
#                 + (3.0 * [[SHIP_SHIELD_FACTOR]] * (Statistic If condition = And [ Source DesignHasPart name = "SH_DEFLECTOR"]))
#                 + (4.5 * [[SHIP_SHIELD_FACTOR]] * (Statistic If condition = And [ Source DesignHasPart name = "SH_PLASMA"]))
#                 + (6.0 * [[SHIP_SHIELD_FACTOR]] * (Statistic If condition = And [ Source DesignHasPart name = "SH_MULTISPEC"]))
#                 + (7.5 * [[SHIP_SHIELD_FACTOR]] * (Statistic If condition = And [ Source DesignHasPart name = "SH_BLACK"]))
# '''

# #include "../macros/misc.macros"
